%Mark Jankauski%
clear all; clc;close all;
tic

%% SUMMARY %%

% The purpose of this script is to demonstrate how to numerically solve the
% equation of motion governing the modal response of the wing.  Wing
% geometry, mode shapes, and natural frequencies are determined using
% Abaqus FEA.  The user inputs are rotation profiles and total time.  The default
% rotation is sinusoidal, but can be changed by the user should other
% rotations be of interest.  The script is heavily commented so the user
% can change parameters as necessary.

% This example is for the aluminum triangular wing used for the
% experimental validation.  Three vibration modes are considered, but could
% be extended to include more modes if necessary.

% The output of this script is the strain at the selected point near the
% wing root as a function of time, and the magnitude of the strain as a
% function of frequency.  

% The most common error for a user unfamiliar with the script is indexing.
% Be sure that the index numbers in the script correspond to the correct
% columns in the .inp and .rpt files generated using FEA


%% INPUT PARAMETERS%%

% In this section, the amplitudes and frequencies of the rotation profiles
% are input.  The script assumes sinusoidal rotations.  This is also where
% modal damping is considered.  Lastly, the total length of the simulation
% and the time step is input.  

N=3;                    %Enter the number of modes considered here%

%The value for 'node' corresponds to the node number that most closely
%matches the physical location where the strain is being evaluated
node=19006;

%Frequency Parameters%
w_a=5;                 %Alpha Rotation Freq, Hz%
w_b=5;                 %Beta Rotataion Freq, Hz%
w_g=5;                 %Gamma Rotation Freq, Hz%

%Rotation parameters%

%For the experimental set-up, rotation amplitudes are measured in quarter
%counts (qc).  The number of qc/revolution is specific to the encoder.  For
%the encoder used, there are 2000 qc/ revolution. Inputs subscripted with a
%_ph correspond to the phase of the sinusoidal rotation.

A=250;                  %Amplitude of roll, main rotation, qc%
A_ph=0;                 %Phase shift of roll rotation, rad%
B=200;                  %Amplitude or pitch, secondary rotation, qc%
B_ph=0;                 %Phase shift of pitch rotation, rad%
G=0;                    %Amplitude of yaw, tertiary rotation, qc%
G_ph=0;                 %Phase shift of yaw rotation, rad%

%Time Parameters
t0=0;                   %The starting time of the simulation
tf=2;                   %The final time of the simulation
h=0.001;                %The time step used for the simulation
td=t0:h:tf;             %Creates a linear array of time for the simulation%

%Natural frequencies%

%The natural frequencies (in Hz) are input here, in ascending order.  These
%are determined via Abaqus FEA and are input manually.  Three modes are
%considered, corresponding to three natural frequencies.  

w_n=[20 146 175.56];  

%Modal damping 

%The modal damping ratios are input here.  The damping ratio may be a
%scalar quantity if all modes are equally damped.  To have non-uniform
%damping ratios, input each damping ratio in the array.  The example is
%currently undamped, for zed=0.

zed=[0 0 0];

%Unit conversions

%Natural frequencies and driving frequencies are converted from Hz to
%rad/sec

w_a=w_a*2*pi; 
w_b=w_b*2*pi; 
w_g=w_g*2*pi;                    
w_n=w_n*2*pi;           

%The rotation amplitudes are converted from qc to radians
A=A*(2*pi)/(2000); B=B*(2*pi)/(2000); G=G*(2*pi)/(2000); 

%The total modal damping term is evaluated
damp=2*zed.*w_n;

%% DATA IMPORT%

% The necessary data from the FEM model is input here.  The files are
% summarized as follows

% Imports the geometry of the wing in x-y-z coordinates.  This data is
% found in the .inp file generated when submitting a job to Abaqus CAE.
% The .inp is converted to a .txt, and all extraneous information in the
% file is deleted
Geo=load('Wing_Geometry.txt'); 

% Imports the nodal mass of the wing.  To determine the nodal mass in FEA,
% all degrees of freedom are constrained and a unit load is applied in the
% downward z-direction, such that the reaction force at each node
% corresponds to the nodal mass.  An .rpt file is then generated reporting
% the reaction force at each node.  This .rpt file is converted to a .txt,
% and all information except the node number and z-component of the
% reaction force is deleted
Node_Mass=load('Nodal_Mass.txt'); 

%Imports the x-y-z coordinates of the first three vibration modes of the
%wing.  The x,y coordinates are treated as zero, as in-plane deformation is
%neglected.  It is important to normalize the mode shapes with respect to
%the mass matrix to remain consistent with formulation.  The Mode files are
%generated by creating a .rpt file after solving the mode shapes in Abaqus.
%Exraneous information is deleted, leaving the modal coordinates

Mode_1=load('Mode1.txt'); 
Mode_2=load('Mode2.txt');
Mode_3=load('Mode3.txt');



%% DATA INDEXING%

%In this section, the data from the Abaqus report files are indexed such
%that they can easily be used in the script.  Any translation or rotation
%of the geometry can be applied in this section.  No rotation is considered
%here, as the CAE model is built in the x-y plane

X_cor=Geo(:,2); Y_cor=Geo(:,3); Z_cor=Geo(:,4); %Creates x,y,z arrays from imported geometry

X_shift=0;          %Non-zero input shifts the wing in the x direction
Y_shift=0;          %Non-zero input shifts the wing in the y direction

%Adds the x,y shifts to the total geometry
X_cor=X_cor+X_shift;
Y_cor=Y_cor+Y_shift;

%Indexes the nodal mass from the imported data.  If the nodal mass was
%found correctly, the absolute value is unnecessary
Node_Mass=abs(Node_Mass(:,2)); 

%Initialize an empty matrix for mode shape data%
Mode=zeros(length(Geo),N); 

%Assembles mode shape data into a single Modal matrix%
Mode(:,1)=Mode_1(:,2); Mode(:,2)=Mode_2(:,2); Mode(:,3)=Mode_3(:,2);


%% INDEX STRAINS

% The modal strain evaluated at the location of interest is indexed here.
% The '5' index corresponds to the strain in the y-direction.  By changing
% this index, you can evaluate the x-component of normal strain or shear
% strain.  The index number depends on how the .rpt files were generated.  

ST1=Mode_1(node,5);
ST2=Mode_2(node,5);
ST3=Mode_3(node,5);


%% PLOT MODE SHAPES

%As a sanity check, this line of code plots the wing geometry in 3D space.
%This helps visualize that any translations or rotations of the CAE model
%were done correctly.

 tri=delaunay (X_cor,Y_cor);
 
    figure()
    trisurf ( tri, X_cor, Y_cor, Z_cor );
    xlabel('x')
    ylabel('y')
    zlabel('z')
    title('Base Wing Geometry')
    xlim([-0.11 0.11])
    ylim([0 0.15])
    view(150,24)
    
    
%% INERTIAL FORCE CENTER CALCULATIONS%

%This section calculates A_k and B_k for each mode shape.  A_k corresponds
%to a vector from the fixed point of rotation to the inertial force center
%of the k^th mode, and B_k corresponds to a 90 degree counter-clockwise
%rotation from A_k.  See the model formulation for more details.  

A_k=zeros(N,3); %Initialize 3xN A_k matrix
B_k=zeros(N,3); %Initialize 3xN B_k matrix

%Create A_k vector matrix%
for i=1:N
    A_k(i,1)=sum(Mode(:,i).*X_cor.*Node_Mass); %Populates I'' field
    A_k(i,2)=sum(Mode(:,i).*Y_cor.*Node_Mass); %Populates J'' field
end

%Create B_k vector matrix
for i=1:N
    B_k(i,1)=sum(Mode(:,i).*-Y_cor.*Node_Mass); %Populates I'' field
    B_k(i,2)=sum(Mode(:,i).*X_cor.*Node_Mass); %Populates J'' field
end


%% ROTATION PARAMETER CALCULATIONS

%The rotation profiles are calculated here from the input rotation
%amplitudes and frequencies.  The time derivatives are calculated
%analytically.  Currently, all rotations are assumed sinusoidal. The user 
%change the form of the driving rotations if desired.  

%Driving roll function and its time derivatives%

Alpha=A*sin(w_a*td+A_ph);
Alpha_D=A*w_a*(cos(w_a*td+A_ph));       %First time derivative of roll
Alpha_DD=-A*w_a^2*(sin(w_a*td+A_ph));   %Second time derivative of roll

%Driving pitch function and its time derivatives%
Beta=B*sin(w_b*td+B_ph);
Beta_D=B*w_b*cos(w_b*td+B_ph);          %First time derivative of pitch
Beta_DD=-B*w_b^2*sin(w_b*td+B_ph);      %Second time derivative of pitch

%Driving yaw function and its time derivatives%
Gamma=G*sin(w_g*td+G_ph);
Gamma_D=G*w_g*cos(w_g*td+G_ph);         %First time derivative of yaw
Gamma_DD=-G*w_g^2*sin(w_g*td+G_ph);     %Second time derivative of yaw

%Instantaneous angular velocity and acceleration vectors are calculated
%from the rotation trajectories.  These expressions are valid even if a
%non-sinusoidal rotation is considered.  

W_a=[(Alpha_D.*cos(Beta))',(Beta_D)',(Alpha_D.*sin(Beta))'];
W_a_d=[(Alpha_DD.*cos(Beta)-Alpha_D.*Beta_D.*sin(Beta))',...
      (Beta_DD)',(Alpha_DD.*sin(Beta)+Alpha_D.*Beta_D.*cos(Beta))'];
  
%% TIME VARYING STIFFNESS%

%The time varying stiffnesses for all natural frequencies are calculated
%here%

Stiffness=zeros(N,length(td)); %Initialize time-varying stiffness matrix%

%Calculates the time-varying stiffness for each mode considered
for i=1:N
    Stiffness(i,:)=w_n(i)^2-((Alpha_D.^2).*(cos(Beta).^2)+(Beta_D.^2));
end


%% EXCITATION PARAMETERS%

%The Euler, Coriolis, Centrifugal and total excitations are calculated
%here.  See the model formulation for a detailed mathematical description
%of each of the excitation terms.  

%Initialize excitation matrices%
Euler_Force=zeros(N,length(td));
Centrifugal_Force=zeros(N,length(td));
Coriolis_Force=zeros(N,length(td));

%Calculate the excitations for each mode shape%
for i=1:N %indexing through the number of mode shapes
    
    for j=1:length(td) %indexing through the time span
    Euler_Force(i,j)=-dot(W_a_d(j,:),(Gamma(j)*A_k(i,:)-B_k(i,:))); %Euler Force calculation
    Centrifugal_Force(i,j)=-dot([0,0,1],W_a(j,:))*dot(W_a(j,:),(A_k(i,:)+Gamma(j)*B_k(i,:))); %Centrifugal force calculation
    Coriolis_Force(i,j)=-2*Gamma_D(j)*dot(W_a(j,:),A_k(i,:)); %Coriolis Force calculation
    end
end

Total_Excitation=Euler_Force+Centrifugal_Force+Coriolis_Force; %Sums all components of the excitation%

%% SOLVE FOR RESPONSES 

%This section is used to calculate each modal response seperately using the
%Wing_Func.  For more details on the Wing_Func, open the function file.  
 [T1 q1]=ode45(@(t,y) Wing_Func(t,y,td,Total_Excitation(1,:),Stiffness(1,:),damp(1)),td,[0,0]);
 [T2 q2]=ode45(@(t,y) Wing_Func(t,y,td,Total_Excitation(2,:),Stiffness(2,:),damp(2)),td,[0,0]);    
 [T3 q3]=ode45(@(t,y) Wing_Func(t,y,td,Total_Excitation(3,:),Stiffness(3,:),damp(3)),td,[0,0]);   

%The modal responses are multiplied by the modal strain to give a physical
%strain value
q1=q1(:,1).*ST1;
q2=q2(:,1).*ST2;
q3=q3(:,1).*ST3;

%Each of the physical strains are summed together to give the total
%physical strain (again, considering 3 mode shapes)
Q=q1+q2+q3;

%The physical strain is multiplied by 10^6 to convert strain to microstrain
Q=Q*10^6;

%This portion of the script takes the FFT and determines the magnitude of
%the strain (in microstrain) as a function of frequency

h=T1(2)-T1(1);          %Determines time step from simulation time             
Fs = h^-1;              % Sampling frequency
T = 1/Fs;               % Sample time
L = length(T1);         % Length of signal
t = (0:L-1)*T;  

NFFT = 2^nextpow2(L);               % Next power of 2 from length of y
Y = fft(Q,NFFT)/L;                  
f = Fs/2*linspace(0,1,NFFT/2+1);    %Sets up the array of frequencies
Y=2*abs(Y(1:NFFT/2+1));             %Determines the magnitude of the strain


%Plots the magnitude of the microstrain as a function of frequency in Hz
figure()
plot(f,Y) 
title('FFT of Strain (Theory)')
xlabel('Frequency (Hz)')
ylabel('|Microstrain|')
xlim([0 20])

%Plots the microstrain versus time signal
figure()
plot(T1,Q)
xlabel('Time (sec)')
ylabel('Strain (Microstrain)')
title('Strain vs. Time')

toc
